@@ -13,7 +13,7 @@ import java.util.*;
 import java.io.File;
 
 import base.drawable.*;
-import logformat.trace.*;
+// import logformat.trace.*;
 import logformat.slog2.*;
 
 public class TraceToSlog2
@@ -340,7 +340,6 @@ public class TraceToSlog2
         } catch ( NumberFormatException numerr ) {
             if ( err_msg.length() > 0 )
                 System.err.println( err_msg.toString() );
-            String idx_order_str = indexOrderStr( idx );
             System.err.println( "Error occurs after option "
                               + argv[ idx-1 ] + ", " + indexOrderStr( idx )
                               + " command line argument.  It needs a number." );
