@@ -10,7 +10,7 @@
 #define FUNCNAME MPID_nem_tcp_finalize
 #undef FCNAME
 #define FCNAME MPIDI_QUOTE(FUNCNAME)
-int MPID_nem_tcp_finalize()
+int MPID_nem_tcp_finalize(void)
 {
     int mpi_errno = MPI_SUCCESS;
     int ret;
@@ -36,12 +36,3 @@ int MPID_nem_tcp_finalize()
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPID_nem_tcp_ckpt_shutdown
-#undef FCNAME
-#define FCNAME MPIDI_QUOTE(FUNCNAME)
-int MPID_nem_tcp_ckpt_shutdown()
-{
-    return MPID_nem_tcp_finalize();
-}
-
