@@ -116,6 +116,8 @@ int main( int argc, char *argv[], char *envp[] )
     MPIE_ProcessInit();
     /* Set a default for the universe size */
     pUniv.size = 64;
+    /* Tell the PMI untility routines that we are the server */
+    PMIU_SetServer();
 
     /* Set defaults for any arguments that are options.  Also check the
        environment for special options, such as debugging.  Set 
@@ -137,8 +139,12 @@ int main( int argc, char *argv[], char *envp[] )
 	/* The MPI process is already running.  We create a simple entry
 	   for a single process rather than creating the process */
 	MPIE_SetupSingleton( &pUniv );
+	/* The handshake to synchronize with the process happens below */
     }
 
+    /* Here is where we would choose the hosts for the job; as gforker
+       uses same machine on which it runs, we don't perform this step */
+
     if (MPIE_Debug) MPIE_PrintProcessUniverse( stdout, &pUniv );
 
     DBG_PRINTF( ("timeout_seconds = %d\n", pUniv.timeout) );
@@ -153,7 +159,7 @@ int main( int argc, char *argv[], char *envp[] )
        init, allowing an MPI process to contact a waiting mpiexec that 
        would serve as a process manager.  This option is not implemented */
     if (getenv("MPIEXEC_USE_PORT")) {
-	s.pmiinfo.portName = (char *)malloc( 1024 );
+	s.pmiinfo.portName = (char *)MPIU_Malloc( 1024 );
 	if (!s.pmiinfo.portName) {
 	    MPIU_Error_printf( "Failed to allocate storage for portName" );
 	}
@@ -172,10 +178,14 @@ int main( int argc, char *argv[], char *envp[] )
     PMIServInit(myspawn,&s);
     s.pmiinfo.pWorld = &pUniv.worlds[0];
     PMISetupNewGroup( pUniv.worlds[0].nProcess, 0 );
+    /* PMISetupNewGroup will create a kvs space */
     MPIE_ForwardCommonSignals();
+    MPIE_IgnoreSigPipe();
+    
     if (!pUniv.fromSingleton) {
 	MPIE_ForkProcesses( &pUniv.worlds[0], envp, mypreamble, &s,
 			    mypostfork, 0, mypostamble, 0 );
+	MPIE_InitForDebugger( &pUniv.worlds[0] );
     }
     else {
 	int newfd;
@@ -197,9 +207,19 @@ int main( int argc, char *argv[], char *envp[] )
 	pState = pUniv.worlds[0].apps->pState;
 	/* FIXME: The following should be a single routine in pmiport */
 	pmiprocess = PMISetupNewProcess( newfd, pState );
-	PMI_Init_port_connection( newfd );
-	PMI_Init_remote_proc( newfd, pmiprocess );
-	printf( "Done with init_remote_proc\n" );
+
+	PMI_InitSingletonConnection( newfd, pmiprocess );
+	/* PMI_Init_port_connection is used with regular (as opposed to
+	   singleton init) and is used to get the pmiid among other 
+	   items.  It isn't needed (and the commands aren't sent by the 
+	   client) in the singleton case */
+	/*	PMI_Init_port_connection( newfd ); */
+
+	/* Init_remote_proc initiates the process of setting the rank and 
+	   debug flags.  This also isn't needed for the singleton init
+	   case */
+	/* PMI_Init_remote_proc( newfd, pmiprocess ); */
+	/* Ready to handle new input */
 	MPIE_IORegister( newfd, IO_READ, PMIServHandleInput, 
 			 pmiprocess );
     }
@@ -269,6 +289,29 @@ int mypreamble( void *data, ProcessState *pState )
        we are using a port, and use the PMI_PORT and ID instead */
     if (usePort) pState->initWithEnv = 0;
     
+    /* Add clique information */
+    {
+	int i, size = pState->app->nProcess;
+	char digits[10], ranks[1024];
+	char key[256];
+	
+	/* Create the string of ranks.  These are ranks in comm_world */
+	ranks[0] = 0;
+	for (i=0; i<size; i++) {
+	    MPIU_Snprintf( digits, sizeof(digits), "%d,", i );
+	    MPIU_Strnapp( ranks, digits, sizeof(ranks) );
+	}
+	/* Remove the trailing comma */
+	if (size > 0) 
+	    ranks[strlen(ranks)-1] = 0;
+	/* Add this to the predefined keys */
+	MPIU_Snprintf( key, sizeof(key), "pmiPrivateLocalRanks_%d", 
+		       pState->wRank );
+	/* printf( "%s = %s\n", key, ranks ); */
+	
+	pmix_preput( key, ranks );
+    }
+
     return 0;
 }
 /* Close one side of each pipe pair and replace stdout/err with the pipes */
@@ -278,7 +321,6 @@ int mypostfork( void *predata, void *data, ProcessState *pState )
 
     IOLabelSetupInClient( &s->labelinfo );
     PMISetupInClient( usePort, &s->pmiinfo );
-/*    setvbuf(stdout,NULL,_IOLBF,0); */
 
     return 0;
 }
